
/* Chrysalide - Outil d'analyse de fichiers binaires
 * op_ret.c - décodage de la sortie d'un appel
 *
 * Copyright (C) 2008-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "opcodes.h"


#include "../operand.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : data   = flux de données à analyser.                         *
*                pos    = position courante dans ce flux. [OUT]               *
*                len    = taille totale des données à analyser.               *
*                addr   = adresse virtuelle de l'instruction.                 *
*                prefix = éventuel(s) préfixe(s) remarqué(s).                 *
*                proc   = architecture ciblée par le désassemblage.           *
*                                                                             *
*  Description : Décode une instruction de type 'retf'.                       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_retf(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */

    result = g_x86_instruction_new(XOP_RETF);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data   = flux de données à analyser.                         *
*                pos    = position courante dans ce flux. [OUT]               *
*                len    = taille totale des données à analyser.               *
*                addr   = adresse virtuelle de l'instruction.                 *
*                prefix = éventuel(s) préfixe(s) remarqué(s).                 *
*                proc   = architecture ciblée par le désassemblage.           *
*                                                                             *
*  Description : Décode une instruction de type 'retf' (16 bits).             *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_retf_imm16(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */

    result = g_x86_instruction_new(XOP_RETF_IMM16);

    if (!x86_read_one_operand(result, data, pos, len, X86_OTP_IMM16))
    {
        g_object_unref(G_OBJECT(result));
        return NULL;
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data   = flux de données à analyser.                         *
*                pos    = position courante dans ce flux. [OUT]               *
*                len    = taille totale des données à analyser.               *
*                addr   = adresse virtuelle de l'instruction.                 *
*                prefix = éventuel(s) préfixe(s) remarqué(s).                 *
*                proc   = architecture ciblée par le désassemblage.           *
*                                                                             *
*  Description : Décode une instruction de type 'retn'.                       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_retn(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */

    result = g_x86_instruction_new(XOP_RETN);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data   = flux de données à analyser.                         *
*                pos    = position courante dans ce flux. [OUT]               *
*                len    = taille totale des données à analyser.               *
*                addr   = adresse virtuelle de l'instruction.                 *
*                prefix = éventuel(s) préfixe(s) remarqué(s).                 *
*                proc   = architecture ciblée par le désassemblage.           *
*                                                                             *
*  Description : Décode une instruction de type 'retn' (16 bits).             *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchInstruction *x86_read_instr_retn_imm16(const bin_t *data, off_t *pos, off_t len, vmpa_t addr, X86Prefix prefix, const GX86Processor *proc)
{
    GArchInstruction *result;               /* Instruction à retourner     */

    result = g_x86_instruction_new(XOP_RETN_IMM16);

    if (!x86_read_one_operand(result, data, pos, len, X86_OTP_IMM16))
    {
        g_object_unref(G_OBJECT(result));
        return NULL;
    }

    return result;

}
