
/* Chrysalide - Outil d'analyse de fichiers binaires
 * die.h - prototypes pour la gestion des entrées renvoyant à des informations de débogage
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _FORMAT_DWARF_DIE_H
#define _FORMAT_DWARF_DIE_H


#include <stdbool.h>


#include "abbrev.h"
#include "dwarf.h"
#include "dwarf_def.h"



/* § 2.1 The Debugging Information Entry (DIE). */
typedef struct _dw_die dw_die;


/* Procède à la lecture de l'en-tête d'une unité de compilation. */
bool build_dwarf_die(GDwarfFormat *, vmpa2t *pos, SourceEndian endian, const dw_compil_unit_header *header, dw_die **);

/* Supprime les éléments mis en place pour une entrée d'info. */
void delete_dwarf_die(dw_die *);

/* Fournit un lien vers l'abréviation de représentation. */
const dw_abbrev *dw_die_get_abbrev(const dw_die *);

/* Fournit un lien vers l'abréviation de représentation. */
const dw_form_value *dw_die_peek_value(const dw_die *, size_t);

/* Fournit un lien vers l'abréviation de représentation. */
const dw_form_value *dw_die_peek_extended_value(const dw_die *, DwarfAttrib, DwarfForm *);

/* Indique si une entrée de débogage possède des enfants. */
bool dw_die_has_children(const dw_die *);

/* Indique si une entrée de débogage possède des enfants. */
void dw_die_append_child(dw_die *, dw_die *);



/* ---------------------------- TRAITEMENT PAR ENSEMBLES ---------------------------- */


/* Procédure appelée pour chaque élément recontré pendant une visite. */
typedef bool (* visit_dies_fc) (const dw_die *, void *);


/* Entame une grande tournée de toutes les entrées présentes. */
bool dw_die_visit(dw_die *, visit_dies_fc, void *);



#endif  /* _FORMAT_DWARF_DIE_H */
